% Black-Derman-Toy interest rate tree fitted to both
% the yield curve and volatilities.
% Reproduces Figure 8.6 in Clewlow and Strickland's book
% "Implementing Derivatives Models".

clc; clear;

% Time (T), number of time steps (N) and increment (dt)
N = 3;
T = 3;
dt = T/N;

% Tenors, rates, volatilities, and discount factors (P).
Tenors = [1 2 3 4];
Rates  = [5 5 5 5]./100;
sigma  = [10 9 8 7]./100;
P = 1./(1+Rates.*dt).^Tenors;

% Initialize the Pu and Pd vectors.
Pu = zeros(1,N+1);
Pd = zeros(1,N+1);

% Find the Pu and Pd vectors
options = optimset('TolFun', 1e-20);
for i=2:N+1
	Pu(i) = fminsearch(@(Pu) findPu(Pu,P(i),Rates(1),sigma(i),dt), 0.9, options);
	Pd(i) = Pu(i)^exp(-2*sigma(i)*sqrt(dt));
end

% Set the objective function 
% 'ABS' (absolute errors) or 'SSE' (squared errors)
ObjFun = 'ABS';

% Return the BDT rates (r), discount factors (d), and forward vols (fsigma)
[r d fsigma] = BuildBDTtree_yields_vols(Rates(1),sigma(1),Pu,Pd,N,dt,ObjFun)


